module vga_driver(
					clk    ,
					rst_n  ,
                    key_en ,
					hys    ,
					vys    ,
                    seg_sel,
                    segment,      
                    led_g  ,
                    led_r  ,
                    led_y  ,
				    lcd_rgb 
               );

parameter     PICTURE_W    =  16 ;
parameter     ROW_W        =   10;
parameter     KEY_W        =   8  ;

parameter     X_WID  = 646;
parameter     Y_HIG  = 484;
parameter     X_MID  = X_WID/2;
parameter     Y_MID  = Y_HIG/2;

parameter     E_WID  = 400;
parameter     E_HIG  = 480;

parameter    TIME_1S   = 60 ;
parameter    TIME_20US = 500;

parameter     WHITE        = 8'b111_111_11;
parameter     RED          = 8'b111_000_00;
parameter     GREEN        = 8'b000_111_00;
parameter     BLUE         = 8'b000_000_11;
parameter     YELLOW       = 8'b111_111_00;
parameter     PURPLE       = 8'b011_000_10;
parameter     CYAN         = 8'b011_111_11;
parameter     PINK         = 8'b111_011_01;
parameter     BLACK        = 8'b000_000_00;
parameter     JUDGE_TIME   = 2;
parameter     RIGHT        = 0;
parameter     DOWN         = 1;
parameter     LEFT         = 2;
parameter     UP           = 3;

parameter     DATA0 = 8'b11000000;
parameter     DATA1 = 8'b11111001;
parameter     DATA2 = 8'b10100100;
parameter     DATA3 = 8'b10110000;
parameter     DATA4 = 8'b10011001;
parameter     DATA5 = 8'b10010010;
parameter     DATA6 = 8'b10000010;
parameter     DATA7 = 8'b11111000;
parameter     DATA8 = 8'b10000000;
parameter     DATA9 = 8'b10010000;

parameter     SEG_NUM = 4;


input                  clk      ;
input                  rst_n    ;
input   [KEY_W-1:0]    key_en   ;

output                 hys      ;
output                 vys      ;
output                 led_g    ;
output                 led_r    ;
output                 led_y    ;

output[3:0]            seg_sel  ;
output[7:0]            segment  ;

output [PICTURE_W-1:0] lcd_rgb;
reg    [PICTURE_W-1:0] lcd_rgb;

reg    [ROW_W-1:0]     h_cnt;
reg    [ROW_W-1:0]     v_cnt;
reg                    hys;
reg                    vys;
reg                    valid_area ;

reg[19:0]              distance   ;
reg                    e_rom_area;
reg[17:0]              e_rom_addr;
reg[ 2:0]              e_rom_addr_low;
reg                    e_rom_addr_hig;
wire[7:0]              e_rom0_data;
wire[7:0]              e_rom1_data;
reg[1:0]               direction    ;
reg[1:0]               direction_ff0;
wire[9:0]              e_x0;
wire[9:0]              e_x1;
wire[9:0]              e_y0;
wire[9:0]              e_y1;
wire[9:0]              wid ;
wire[9:0]              hig ;
wire[9:0]              wid_size ;
wire[9:0]              hig_size ;
reg                    e_sel;
reg [9:0]              x_p;
reg [9:0]              y_p;
wire[9:0]              x;
wire[9:0]              y;

reg                    vys_tmp;
reg                    hys_tmp;
reg                    valid_area_ff0;
reg                    e_rom_area_ff0   ;
reg[3:0]               size;
reg[3:0]               size_ff0;
wire                   judge_rig;
wire                   judge_err;
reg[3:0]               judge_rig_cnt;
reg[3:0]               judge_err_cnt;
reg                    direction_en ;
reg[1:0]               direction_random;

reg                    led_g           ;
reg                    led_r           ;
reg                    led_y           ;

reg   [3:0]            seg_sel  ;
reg   [7:0]            segment  ;
reg [3:0]              err_cnt_shi;
reg [3:0]              err_cnt_ge ;
reg [3:0]              rig_cnt_shi;
reg [3:0]              rig_cnt_ge ;
wire                   flag_20us ;
reg  [9:0]             count_20us;
reg[7:0]               sel_result;
reg[2:0]               sel_cnt ;
reg[5:0]               led_y_cnt;

always@(posedge clk or negedge rst_n)begin
	if(!rst_n)begin
      	h_cnt <= 10'd0;
	 end
    else if(h_cnt == 10'd799)begin
        h_cnt <= 10'd0;
     end
    else begin
	    h_cnt <= h_cnt+1'b1;
     end
 end

 always@(posedge clk or negedge rst_n)begin
	if(!rst_n) begin
       v_cnt <= 10'd0;
    end
    else if(h_cnt == 10'd799)begin
        if(v_cnt==524)
            v_cnt <= 0;
        else
            v_cnt <= v_cnt+1;
    end
 end
	

always@(posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        hys_tmp <= 0;
    end
    else if(h_cnt==10'd95)begin
        hys_tmp <= 1'b1;
    end
    else if(h_cnt==10'd799)begin
        hys_tmp <= 1'b0;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        hys <= 0;
    end
    else begin
        hys <= hys_tmp;
    end
end

	


always@(posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        vys_tmp <= 1'b0;
    end
    else if(v_cnt<10'd2)begin
        vys_tmp <= 1'b0;
    end
    else begin
        vys_tmp <= 1'b1;
    end
end    

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        vys <= 1'b0;
    end
    else begin
        vys <= vys_tmp;
    end
end


always  @(*)begin
    valid_area = h_cnt >=141 && h_cnt <=786 && v_cnt >=32 && v_cnt <=515;
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        valid_area_ff0    <= 1'b0;
        e_rom_area_ff0    <= 1'b0;
    end
    else begin
        valid_area_ff0    <= valid_area;
        e_rom_area_ff0    <= e_rom_area   ;
    end
end



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        size <= 0;
    end
    else if(judge_rig_cnt==JUDGE_TIME-1 && judge_rig) begin
        if(size!=7)
            size <= size + 1;
    end
    else if(judge_err_cnt==JUDGE_TIME-1 && judge_err)begin
        if(size!=0)
            size <= size - 1;
    end
end


assign judge_rig = (key_en[0] && direction==  UP)||
                   (key_en[1] && direction==DOWN)||
                   (key_en[2] && direction==LEFT)||
                   (key_en[3] && direction==RIGHT);

assign judge_err  = key_en!=0 && (judge_rig==1'b0);

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        judge_rig_cnt <= 0;
    end
    else if(judge_rig) begin
        if(judge_rig_cnt==JUDGE_TIME-1)
            judge_rig_cnt <= 0;
        else
            judge_rig_cnt <= judge_rig_cnt + 1;
    end
    else if(judge_err)begin
        judge_rig_cnt <= 0;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        judge_err_cnt <= 0;
    end
    else if(judge_err) begin
        if(judge_err_cnt==JUDGE_TIME-1)
            judge_err_cnt <= 0;
        else
            judge_err_cnt <= judge_err_cnt + 1;
    end
    else if(judge_rig)begin
        judge_err_cnt <= 0;
    end
end



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        direction_en <= 1'b0;
    end
    else if(key_en[3:0]!=0) begin
        direction_en <= 1'b1;
    end
    else begin
        direction_en <= 1'b0;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        direction_random <= 2'b00;
    end
    else begin
        direction_random <= direction_random + 1;
    end
end



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        direction <= RIGHT;
    end
    else if(direction_en) begin
        direction <= direction_random;
    end
end



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        direction_ff0 <= 2'b00;
    end
    else if(h_cnt==0 && v_cnt==0) begin
        direction_ff0 <= direction;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        size_ff0 <= 0;
    end
    else if(h_cnt==0 && v_cnt==0) begin
        size_ff0 <= size;
    end
end


assign wid = (direction_ff0==RIGHT || direction_ff0==LEFT)?E_WID:E_HIG;
assign hig = (direction_ff0==RIGHT || direction_ff0==LEFT)?E_HIG:E_WID;

assign wid_size = wid >> size_ff0;
assign hig_size = hig >> size_ff0;



assign   e_x0 = ((X_MID-wid_size[9:1]) + 141);
assign   e_x1 = ((X_MID+wid_size[9:1]) + 141);
assign   e_y0 = ((Y_MID-hig_size[9:1]) +  32);
assign   e_y1 = ((Y_MID+hig_size[9:1]) +  32);





always  @(*)begin
    e_rom_area = h_cnt >=e_x0 && h_cnt < e_x1 && v_cnt >= e_y0 && v_cnt < e_y1;
end



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        lcd_rgb <= 0;
    end
    else if(valid_area_ff0)begin
        if(e_rom_area_ff0)
            lcd_rgb <= {16{e_sel}};
        else begin
            lcd_rgb <= {16{1'b1}};
        end
    end
    else begin
        lcd_rgb <= 0;
    end
end


assign x = (h_cnt-e_x0)<<size;
assign y = (v_cnt-e_y0)<<size;

always  @(*)begin
    if(direction_ff0==RIGHT)
        x_p = x;
    else if(direction_ff0==DOWN)
        x_p = y;
    else if(direction_ff0==LEFT)
        x_p = E_WID-x;
    else
        x_p = E_WID-y;
end

always  @(*)begin
    if(direction_ff0==RIGHT)
        y_p = y;
    else if(direction_ff0==DOWN)
        y_p = Y_HIG-x;
    else if(direction_ff0==LEFT)
        y_p = y;
    else
        y_p = x;
end


always  @(*)begin
    e_rom_addr = (x_p + y_p*E_WID);
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        e_rom_addr_low <= 0;
        e_rom_addr_hig <= 0;
    end
    else begin
        e_rom_addr_low <= e_rom_addr[2:0];
        e_rom_addr_hig <= e_rom_addr[17];
    end
end


always  @(*)begin
    if(e_rom_addr_hig==1'b0)
        e_sel = ~e_rom0_data[7-e_rom_addr_low];
    else
        e_sel = ~e_rom1_data[7-e_rom_addr_low];
end



e0_rom u_e0_rom(
	           .address (e_rom_addr[16:3]),
	           .clock   (clk             ),
	           .q       (e_rom0_data      ));

e1_rom u_e1_rom(
	           .address (e_rom_addr[16:3]),
	           .clock   (clk             ),
	           .q       (e_rom1_data      ));


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_g <= 1'b1;
    end
    else if(judge_rig) begin
        led_g <= 1'b0;
    end
    else if(judge_err)begin
        led_g <= 1'b1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_r <= 1'b1;
    end
    else if(judge_err) begin
        led_r <= 1'b0;
    end
    else if(judge_rig)begin
        led_r <= 1'b1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_y <= 1'b1;
    end
    else if(judge_rig || judge_err) begin
        led_y <= 1'b0;
    end
    else if(led_y_cnt==TIME_1S-1 && h_cnt==799 && v_cnt==524)begin
        led_y <= 1'b1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_y_cnt <= 0;
    end
    else if(led_y==1'b0 && h_cnt==799 && v_cnt==524) begin
        if(led_y_cnt==TIME_1S-1)
            led_y_cnt <= 0;
        else
            led_y_cnt <= led_y_cnt + 1;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        err_cnt_ge <= 0;
    end
    else if(judge_err && (err_cnt_shi!=9 || err_cnt_ge!=9))begin
        if(err_cnt_ge==9)
            err_cnt_ge <= 0;
        else
            err_cnt_ge <= err_cnt_ge + 1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        err_cnt_shi <= 0;
    end
    else if(judge_err && err_cnt_ge==9 && err_cnt_shi!=9)begin
        err_cnt_shi <= err_cnt_shi + 1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        rig_cnt_ge <= 0;
    end
    else if(judge_rig && (rig_cnt_shi!=9 || rig_cnt_ge!=9))begin
        if(rig_cnt_ge==9)
            rig_cnt_ge <= 0;
        else
            rig_cnt_ge <= rig_cnt_ge + 1;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        rig_cnt_shi <= 0;
    end
    else if(judge_rig && rig_cnt_ge==9 && rig_cnt_shi!=9)begin
        rig_cnt_shi <= rig_cnt_shi + 1;
    end
end


assign  flag_20us = count_20us==TIME_20US-1;

always@(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        count_20us<=26'b0;
    end
    else if(flag_20us) begin
        count_20us<=0;
    end
    else begin
        count_20us<=count_20us+1'b1;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        sel_cnt <= 0;
    end
    else if(flag_20us) begin
        if(sel_cnt==SEG_NUM-1)
            sel_cnt <= 0;
        else
            sel_cnt <= sel_cnt + 1;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        seg_sel <= {SEG_NUM{1'b1}};
    end
    else begin
        seg_sel <= ~(1'b1 << sel_cnt);
    end
end


always  @(*)begin
    if(sel_cnt==0)begin
        sel_result <= rig_cnt_ge;
    end
    else if(sel_cnt==1)begin
        sel_result <= rig_cnt_shi;
    end
    else if(sel_cnt==2)begin
        sel_result <= err_cnt_ge;
    end
    else begin
        sel_result <= err_cnt_shi;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        segment <= DATA0;
    end
    else begin
        case(sel_result)
            0: segment <= DATA0;
            1: segment <= DATA1;
            2: segment <= DATA2;
            3: segment <= DATA3;
            4: segment <= DATA4;
            5: segment <= DATA5;
            6: segment <= DATA6;
            7: segment <= DATA7;
            8: segment <= DATA8;
            9: segment <= DATA9;
            default : segment <= 8'hff;
        endcase
    end
end



endmodule
